# -*- coding: utf-8 -*- 
# @Time    : 05/10/2024 
# @Author  : nzooherd 
# @File    : bytes_util.py 
# @Software: Visual Studio Code


import typing

def int2byte(num: int, length: int=1) -> bytes:
    return num.to_bytes(length, byteorder='big', signed=False)

def byte2int(data: bytes) -> int:
    return int.from_bytes(data, byteorder='big', signed=False)

def set_to_bytes(input_set: typing.Set[int]) -> bytes:
    """
    Converts a set of integers between 0 and 63 to a bytes object of length 8.
    If an integer is present in the set, the corresponding bit is set to 1, otherwise 0.
    """
    # 初始化一个 8 字节的 bytes 对象，所有位都设置为 0
    result = bytes(8)

    for num in input_set:
        # 确保输入数字在 0 ~ 63 之间
        if not 0 <= num < 64:
            raise ValueError(f"Invalid input: {num} is not between 0 and 63")

        # 计算应该设置的字节索引和位索引
        byte_index = num // 8
        bit_index = num % 8

        # 将对应位设置为 1
        result = result[:byte_index] + bytes([result[byte_index] | (1 << bit_index)]) + result[byte_index + 1:]

    return result

def bytes_to_set(input_bytes: bytes) -> set:
    """
    Conve    If a bit is set to 1 in the bytes object, the corresponding integer is included in the set.
    """
    if len(input_bytes) != 8:
        raise ValueError("Input must be a bytes object of length 8")

    result = set()
    for byte_index, byte_value in enumerate(input_bytes):
        for bit_index in range(8):
            if byte_value & (1 << bit_index):
                result.add(byte_index * 8 + bit_index)

    return result


class CallbackBytesRequest:

    def __init__(self, api_path: str, params_size: typing.List[int]) -> None:
        # 4 bytes message_id
        self.message_id = 0
        # 8 bytes api_path
        self.api_path = api_path
        if (len(api_path) > 8):
            raise Exception("api_path length must be less than 8")

        self.params_size = params_size
        self.params: typing.List[bytes] = [] 

    def to_byte(self) -> bytes:
        return int2byte(self.message_id, 4) + self.api_path.encode("utf-8").ljust(8, b'\0') + b''.join(self.params)


    @classmethod
    def from_byte(cls, params_name: typing.List[str], params_size: typing.List[int], data: bytes):
        message_id = int.from_bytes(data[0:4], byteorder='big', signed=False)
        api_path = data[4:4 + 8].decode("utf-8")

        instance = cls(api_path, params_size)
        instance.message_id = message_id
        start_index = 12
        for i in range(len(params_size)):
            setattr(instance, params_name[i], data[start_index :start_index + params_size[i]])
            start_index += params_size[i]

        return instance

    @classmethod
    def decode_api_path(cls, data: bytes):
        return data[4:4 + 8].decode("utf-8")
